-------------------------------------------------------------------------------
-- Add Finalization - A library for adding finalization to tagged types.
-- Copyright (C) 2003 - 2004  Anders Gidenstam
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
-- As a special exception, if other files instantiate generics from this
-- unit, or you link this unit with other files to produce an executable,
-- this unit does not by itself cause the resulting executable to be
-- covered by the GNU General Public License. This exception does not
-- however invalidate any other reasons why the executable file might be
-- covered by the GNU Public License.
-------------------------------------------------------------------------------
--                              -*- Mode: Ada -*-
-- Filename        : add_finalization-anywhere-to_limited_uncontrolled.adb
-- Description     : Add finalization to any tagged type anywhere.
-- Author          : Anders Gidenstam
-- Created On      : Thu Nov  6 18:41:46 2003
-- $Id: add_finalization-anywhere-to_limited_uncontrolled.adb,v 1.3.2.1 2004/09/15 21:55:39 anders Exp $
-------------------------------------------------------------------------------

with Ada.Text_IO;

package body Add_Finalization.Anywhere.To_Limited_Uncontrolled is

   function To_Controlled is new Ada.Unchecked_Conversion (Object_Ref,
                                                           Controlled_Access);

   ----------------------------------------------------------------------------
   procedure Raw_Initialize (Object : in     Object_Ref) is
   begin
      --  Dispatching call to primitive operation.
      Initialize (Limited_Controlled'Class (To_Controlled (Object).all));
   end Raw_Initialize;

   procedure Raw_Finalize (Object : in     Object_Ref) is
   begin
      --  Dispatching call to primitive operation.
      Finalize (Limited_Controlled'Class (To_Controlled (Object).all));
   end Raw_Finalize;

   ----------------------------------------------------------------------------
   --  Default implementations for Limited_Controlled.
   --  Note that Limited_Controlled is not a descendant of
   --  Ada.Finalization./Limited_/Controlled.
   procedure Initialize (Object : in out Limited_Controlled) is
   begin
      if Test_Mode then
         Ada.Text_IO.Put_Line
           ("Add_Finalization.Anywhere.To_Limited_Uncontrolled.Initialize");
      else
         null;
      end if;
   end Initialize;
   procedure Finalize (Object : in out Limited_Controlled) is
   begin
      if Test_Mode then
         Ada.Text_IO.Put_Line
           ("Add_Finalization.Anywhere.To_Limited_Uncontrolled.Finalize");
      else
         null;
      end if;
   end Finalize;

end Add_Finalization.Anywhere.To_Limited_Uncontrolled;
