-------------------------------------------------------------------------------
-- Add Finalization - A library for adding finalization to tagged types.
-- Copyright (C) 2003 - 2004  Anders Gidenstam
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
-- As a special exception, if other files instantiate generics from this
-- unit, or you link this unit with other files to produce an executable,
-- this unit does not by itself cause the resulting executable to be
-- covered by the GNU General Public License. This exception does not
-- however invalidate any other reasons why the executable file might be
-- covered by the GNU Public License.
-------------------------------------------------------------------------------
--                              -*- Mode: Ada -*-
-- Filename        : add_finalization-anywhere-to_limited_uncontrolled.ads
-- Description     : Add finalization to any tagged type anywhere.
-- Author          : Anders Gidenstam
-- Created On      : Thu Nov  6 18:43:34 2003
-- $Id: add_finalization-anywhere-to_limited_uncontrolled.ads,v 1.4.2.4 2004/09/15 22:11:03 anders Exp $
-------------------------------------------------------------------------------

generic
   type Uncontrolled (<>) is abstract tagged limited private;
package Add_Finalization.Anywhere.To_Limited_Uncontrolled is

   type Limited_Controlled is abstract new Uncontrolled with private;
   --  Note: If Uncontrolled or a descendant of Limited_Controlled
   --  contains controlled components these might have been be finalized
   --  /before/ Finalize is called for this object.
   --  See LRM 7.6.1 and remember that in this case Finalize for
   --  Limited_Controlled will be called when a controlled component
   --  of if is finalized.

   procedure Initialize (Object : in out Limited_Controlled);
   --  Warning: Initialize will never be called!
   procedure Finalize   (Object : in out Limited_Controlled);

private
   type Controlled_Access is access all Limited_Controlled'Class;

   function To_Raw is new Ada.Unchecked_Conversion (Controlled_Access,
                                                    Object_Ref);

   type Local_Operation is access procedure (Object : Object_Ref);
   function To_Operation is new Ada.Unchecked_Conversion (Local_Operation,
                                                          Operation);

   procedure Raw_Initialize (Object : in     Object_Ref);
   procedure Raw_Finalize   (Object : in     Object_Ref);

   type Force_Limited is limited null record;

   type Limited_Controlled is abstract new Uncontrolled with
      record
         Controller : Limited_Controlled_Component :=
             (Ada.Finalization.Controlled with
              --  We might have a problem here!
              --  It is quite possibly not legal to use
              --  Limited_Controlled'Unchecked_Access in this way here.
              --  GNAT does seem to what we want, though, but there are
              --  no guarantees.
              Object     => To_Raw (Limited_Controlled'Unchecked_Access),
              Initialize => To_Operation (Raw_Initialize'Access),
              Finalize   => To_Operation (Raw_Finalize'Access));

         --  Really ugly way to make sure we don't extend an unlimited types.
         --  Do we need to worry? Yes, the self reference will become messed
         --  up otherwise.
         Dummy : Force_Limited;
      end record;

end Add_Finalization.Anywhere.To_Limited_Uncontrolled;
