-------------------------------------------------------------------------------
-- Add Finalization - A library for adding finalization to tagged types.
-- Copyright (C) 2003 - 2004  Anders Gidenstam
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
-- As a special exception, if other files instantiate generics from this
-- unit, or you link this unit with other files to produce an executable,
-- this unit does not by itself cause the resulting executable to be
-- covered by the GNU General Public License. This exception does not
-- however invalidate any other reasons why the executable file might be
-- covered by the GNU Public License.
-------------------------------------------------------------------------------
--                              -*- Mode: Ada -*-
-- Filename        : add_finalization-anywhere.ads
-- Description     : Add finalization to any tagged type anywhere.
-- Author          : Anders Gidenstam
-- Created On      : Thu Nov  6 21:23:30 2003
-- $Id: add_finalization-anywhere.ads,v 1.3.2.1 2004/09/12 19:22:48 anders Exp $
-------------------------------------------------------------------------------

with Ada.Finalization;
with System.Storage_Elements;
with Ada.Unchecked_Conversion;

package Add_Finalization.Anywhere is

private

   --  Set Test_Mode to True to enable debugging output.
   Test_Mode : constant Boolean := True;

   type Object_Ref is new System.Storage_Elements.Integer_Address;
   type Operation is access procedure (Object : Object_Ref);

   --  The actual workhorse of this library.
   --  It will be added as a component of the type we want to controll
   --  and hackish tricks will be used to store a reference to the
   --  object we want to controll in it.
   type Limited_Controlled_Component is
     new Ada.Finalization.Controlled with
      record
         Object     : Object_Ref;
         Initialize : Operation;
         Finalize   : Operation;
      end record;
   procedure Initialize (Object : in out Limited_Controlled_Component);
   procedure Finalize   (Object : in out Limited_Controlled_Component);

end Add_Finalization.Anywhere;
