-------------------------------------------------------------------------------
-- Add Finalization - A library for adding finalization to tagged types.
-- Copyright (C) 2003 - 2004  Anders Gidenstam
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
-- As a special exception, if other files instantiate generics from this
-- unit, or you link this unit with other files to produce an executable,
-- this unit does not by itself cause the resulting executable to be
-- covered by the GNU General Public License. This exception does not
-- however invalidate any other reasons why the executable file might be
-- covered by the GNU Public License.
-------------------------------------------------------------------------------
--                              -*- Mode: Ada -*-
-- Filename        : test_finalization_anywhere.adb
-- Description     : Test of finalization anywhere package.
-- Author          : Anders Gidenstam
-- Created On      : Thu Nov  6 11:49:11 2003
-- $Id: test_finalization_anywhere.adb,v 1.3.2.1 2004/09/14 23:27:23 anders Exp $
-------------------------------------------------------------------------------

with Ada.Text_IO;
with Ada.Unchecked_Conversion;
with System.Storage_Elements;

with Add_Finalization.Anywhere.To_Limited_Uncontrolled;

procedure Test_Finalization_Anywhere is

   ----------------------------------------------------------------------------
   type Local is tagged limited
      record
         Dummy : Integer := 0;
      end record;

   package Finalization is
      new  Add_Finalization.Anywhere.To_Limited_Uncontrolled (Local);

   ----------------------------------------------------------------------------
   type Controlled_Local is new Finalization.Limited_Controlled with
      record
         Bepa : Integer := 12;
      end record;
   procedure Initialize (Object : in out Controlled_Local);
   procedure Finalize   (Object : in out Controlled_Local);

   procedure Initialize (Object : in out Controlled_Local) is
      use System.Storage_Elements;
   begin
      Ada.Text_IO.Put_Line ("Controlled_Local: Initialized " &
                            Integer_Address'Image
                            (To_Integer (Object'Address)) & ".");
   end Initialize;
   procedure Finalize   (Object : in out Controlled_Local) is
      use System.Storage_Elements;
   begin
      Ada.Text_IO.Put_Line ("Controlled_Local: Finalized " &
                            Integer_Address'Image
                            (To_Integer (Object'Address)) & "." &
                            " (Dummy => " & Integer'Image (Object.Dummy) &
                            ", Bepa => " & Integer'Image (Object.Bepa) & ")");
   end Finalize;

   ----------------------------------------------------------------------------
   type Controlled_Local2 is new Controlled_Local with
      record
         Cepa : Integer := 74;
      end record;
   procedure Finalize   (Object : in out Controlled_Local2);

   procedure Finalize   (Object : in out Controlled_Local2) is
      use System.Storage_Elements;
   begin
      Ada.Text_IO.Put_Line ("Controlled_Local2: Finalized " &
                            Integer_Address'Image
                            (To_Integer (Object'Address)) & "." &
                            " (Dummy => " & Integer'Image (Object.Dummy) &
                            ", Bepa => " & Integer'Image (Object.Bepa) &
                            ", Cepa => " & Integer'Image (Object.Cepa) & ")");
   end Finalize;

   LC  : Controlled_Local;
   LC2 : Controlled_Local;
   LC3 : Controlled_Local2;

begin
   LC.Dummy := 1;
   LC2.Dummy := 2;

   Ada.Text_IO.Put_Line ("Finalization should start now!");
end Test_Finalization_Anywhere;
